@@ -18,7 +18,10 @@ use crate::{
         Extensions, Method, Uri, Version,
     },
     route::PathParams,
-    web::RemoteAddr,
+    web::{
+        headers::{Header, HeaderMapExt},
+        RemoteAddr,
+    },
     RequestBody,
 };
 
@@ -257,12 +260,9 @@ impl Request {
     #[cfg_attr(docsrs, doc(cfg(feature = "cookie")))]
     #[inline]
     pub fn cookie(&self) -> &CookieJar {
-        match &self.state.cookie_jar {
-            Some(cookie_jar) => cookie_jar,
-            None => panic!(
-                "To use the `CookieJar` extractor, the `CookieJarManager` middleware is required."
-            ),
-        }
+        self.state.cookie_jar.as_ref().expect(
+            "To use the `CookieJar` extractor, the `CookieJarManager` middleware is required.",
+        )
     }
 
     /// Sets the body for this request.
@@ -346,7 +346,7 @@ impl RequestBuilder {
         Self { version, ..self }
     }
 
-    /// Appends a header to this response builder.
+    /// Appends a header to this request.
     #[must_use]
     pub fn header<K, V>(mut self, key: K, value: V) -> Self
     where
@@ -361,7 +361,14 @@ impl RequestBuilder {
         self
     }
 
-    /// Sets the `Content-Type` header on the response.
+    /// Inserts a typed header to this request.
+    #[must_use]
+    pub fn typed_header<T: Header>(mut self, header: T) -> Self {
+        self.headers.typed_insert(header);
+        self
+    }
+
+    /// Sets the `Content-Type` header to this request.
     #[must_use]
     pub fn content_type(mut self, content_type: &str) -> Self {
         if let Ok(value) = content_type.try_into() {
