@@ -4,11 +4,14 @@ use std::{
     fmt::{self, Debug, Formatter},
 };
 
+use headers::HeaderMapExt;
+
 use crate::{
     http::{
         header::{self, HeaderMap, HeaderName, HeaderValue},
         Extensions, StatusCode, Version,
     },
+    web::headers::Header,
     Body, Error,
 };
 
@@ -259,6 +262,13 @@ impl ResponseBuilder {
         self
     }
 
+    /// Inserts a typed header to this response.
+    #[must_use]
+    pub fn typed_header<T: Header>(mut self, header: T) -> Self {
+        self.headers.typed_insert(header);
+        self
+    }
+
     /// Sets the `Content-Type` header on the response.
     #[must_use]
     pub fn content_type(mut self, content_type: &str) -> Self {
